@@ -20,6 +20,7 @@ class Options:
     PER_MODULE_OPTIONS = {
         "ignore_missing_imports",
         "follow_imports",
+        "follow_imports_for_stubs",
         "disallow_any_generics",
         "disallow_any_unimported",
         "disallow_any_expr",
@@ -38,6 +39,8 @@ class Options:
         "ignore_errors",
         "strict_boolean",
         "no_implicit_optional",
+        "always_true",
+        "always_false",
         "strict_optional",
         "disallow_untyped_decorators",
     }
@@ -52,7 +55,10 @@ class Options:
 
         # -- build options --
         self.build_type = BuildType.STANDARD
-        self.python_version = defaults.PYTHON3_VERSION
+        self.python_version = sys.version_info[:2]  # type: Tuple[int, int]
+        # The executable used to search for PEP 561 packages. If this is None,
+        # then mypy does not search for PEP 561 packages.
+        self.python_executable = sys.executable  # type: Optional[str]
         self.platform = sys.platform
         self.custom_typing_module = None  # type: Optional[str]
         self.custom_typeshed_dir = None  # type: Optional[str]
@@ -60,6 +66,9 @@ class Options:
         self.report_dirs = {}  # type: Dict[str, str]
         self.ignore_missing_imports = False
         self.follow_imports = 'normal'  # normal|silent|skip|error
+        # Whether to respect the follow_imports setting even for stub files.
+        # Intended to be used for disabling specific stubs.
+        self.follow_imports_for_stubs = False  # type: bool
 
         # disallow_any options
         self.disallow_any_generics = False
@@ -127,6 +136,12 @@ class Options:
         # Don't assume arguments with default values of None are Optional
         self.no_implicit_optional = False
 
+        # Variable names considered True
+        self.always_true = []  # type: List[str]
+
+        # Variable names considered False
+        self.always_false = []  # type: List[str]
+
         # Use script name instead of __main__
         self.scripts_are_modules = False
 
@@ -136,14 +151,16 @@ class Options:
         # Write junit.xml to given file
         self.junit_xml = None  # type: Optional[str]
 
-        # Caching options
-        self.incremental = False
+        # Caching and incremental checking options
+        self.incremental = True
         self.cache_dir = defaults.CACHE_DIR
         self.debug_cache = False
         self.quick_and_dirty = False
         self.skip_version_check = False
         self.fine_grained_incremental = False
+        # Include fine-grained dependencies in written cache files
         self.cache_fine_grained = False
+        # Read cache files in fine-grained incremental mode (cache must include dependencies)
         self.use_fine_grained_cache = False
 
         # Paths of user plugins
@@ -218,5 +235,5 @@ class Options:
         # that package's __init__.)
         return pattern.match(module) is not None or pattern.match(module + '.') is not None
 
-    def select_options_affecting_cache(self) -> Mapping[str, bool]:
+    def select_options_affecting_cache(self) -> Mapping[str, object]:
         return {opt: getattr(self, opt) for opt in self.OPTIONS_AFFECTING_CACHE}
